@@ -829,6 +829,7 @@ SEXP GPB_CreateREModel_R(SEXP ndata,
 	SEXP cover_tree_radius,
 	SEXP ind_points_selection,
 	SEXP likelihood,
+	SEXP likelihood_additional_param,
 	SEXP matrix_inversion_method,
 	SEXP seed) {
 	SEXP ret;
@@ -877,6 +878,7 @@ SEXP GPB_CreateREModel_R(SEXP ndata,
 		Rf_asReal(cover_tree_radius),
 		ind_points_selection_ptr,
 		likelihood_ptr,
+		Rf_asReal(likelihood_additional_param),
 		matrix_inversion_method_ptr,
 		Rf_asInteger(seed),
 		&handle));
@@ -1316,7 +1318,7 @@ static const R_CallMethodDef CallEntries[] = {
   {"LGBM_BoosterSaveModel_R"          , (DL_FUNC)&LGBM_BoosterSaveModel_R          , 4},
   {"LGBM_BoosterSaveModelToString_R"  , (DL_FUNC)&LGBM_BoosterSaveModelToString_R  , 4},
   {"LGBM_BoosterDumpModel_R"          , (DL_FUNC)&LGBM_BoosterDumpModel_R          , 3},
-  {"GPB_CreateREModel_R"              , (DL_FUNC)&GPB_CreateREModel_R              , 26},
+  {"GPB_CreateREModel_R"              , (DL_FUNC)&GPB_CreateREModel_R              , 27},
   {"GPB_REModelFree_R"                , (DL_FUNC)&GPB_REModelFree_R                , 1},
   {"GPB_SetOptimConfig_R"             , (DL_FUNC)&GPB_SetOptimConfig_R             , 28},
   {"GPB_OptimCovPar_R"                , (DL_FUNC)&GPB_OptimCovPar_R                , 3},
